//*********************************************************************
//* I2C Control on Raspberry for external Eeprom
//*********************************************************************
//*
//* This Program should communicate with I2C devices in general
//* At the moment it tries to communicate with a serial I2C Eeprom 32KB x 8Bit (Microchip : 24LC256)
//*
//********************************************************************
//* Original Program and still parts of the program from www.teso.de
//* Written on a Raspberry 3 (stretch)
//* Lazarus : v1.6.2
//* Autor   : M.\\/eber
//* Version : 0.01  Date : 29.09.2017
//***********************************************************************
//* How to use :
//* place two Eeprom Microchip : 24LC256 to the I2C bus. Address them to $52 and $53
//* With this programm you should be able to choose a an addess on the Eeprom and write a byte in it.
//* Reading any adresses should work too.
//*************************************************************************3
//* Does only half work up to now.  :-(
//* Writes to the wrong I2C address



unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  Spin, Process, I2C;

type

  { TForm1 }

  TForm1 = class(TForm)
    btnScanBus: TButton;
    btnI2CFree: TButton;
    btnSetDevice: TButton;
    btnCreateDevice: TButton;
    btn52Read: TButton;
    btn52Write: TButton;
    btnI2CDestroy: TButton;
    edtCtrByteWr52: TEdit;
    edtAddrHiBinWr: TEdit;
    edtAddrLoBinWr: TEdit;
    edtDataBinWr: TEdit;
    edtCtrByteRd52: TEdit;
    edtAddrHiBinRd: TEdit;
    edtAddrLoBinRd: TEdit;
    edtDataBinRd: TEdit;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Memo1: TMemo;
    Memo2Send: TMemo;
    Memo3Read: TMemo;
    rbI2C_1: TRadioButton;
    rbI2C_0: TRadioButton;
    spnedtAddressHi: TSpinEdit;
    spnedtAddressLo: TSpinEdit;
    spnedtData: TSpinEdit;
    procedure btn52ReadClick(Sender: TObject);
    procedure btn52WriteClick(Sender: TObject);
    procedure btn53ReadClick(Sender: TObject);
    procedure btn53WriteClick(Sender: TObject);
    procedure btnI2CDestroyClick(Sender: TObject);
    procedure btnI2CFreeClick(Sender: TObject);
    procedure btnCreateDeviceClick(Sender: TObject);

    procedure btnScanBusClick(Sender: TObject);
    procedure btnSetDeviceClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Label1Click(Sender: TObject);
    procedure Memo1Change(Sender: TObject);
    procedure rbI2C_1Change(Sender: TObject);
    procedure spnedtAddressHiChange(Sender: TObject);
    procedure spnedtAddressLoChange(Sender: TObject);
    procedure spnedtDataChange(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }

    i2c     : TI2C;
    strSlaveAddress   : string;
    iZiel   : byte;
    sDevice : string;
    sDummy  : string;
    buf      : array [1..9] of byte;

    bufByteWrite      : array [1..4] of byte;
    bufByteRead      : array [1..3] of byte;
    bufByteData      : array [1..1] of byte;

    CtrlByteChip52Write : Byte;  // 1010 010 0    Address Hex 52
    CtrlByteChip52Read  : Byte;  // 1010 010 1

    CtrlByteChip53Write : Byte;  // 1010 011 0    Address Hex 53
    CtrlByteChip53Read  : Byte;  // 1010 011 1

    AddressLoByte : Byte;
    AddressHiByte : Byte;
    DataByte : Byte;
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }




procedure TForm1.btnI2CFreeClick(Sender: TObject);
begin
       i2c.Free;

 //    Form1.Close;
end;

procedure TForm1.btn52ReadClick(Sender: TObject);
begin
  // Write Address to I2C
  bufByteRead[1] := CtrlByteChip52Write;
  bufByteRead[2] := AddressHiByte;
  bufByteRead[3] := AddressLoByte;

  i2c.writeBuf(bufByteRead);
  // Memo2Send.Lines.Add(binstr(bufByteRead[1],8)+binstr(bufByteRead[2],8)+bufByteRead[3],8));
  Memo2Send.Lines.Add(binstr(bufByteRead[1],8)+binstr(bufByteRead[2],8));

  // Read Data from I2C
  i2c.readBuf(bufByteData);

  edtDataBinRd.text := binstr(bufByteData[1],8);
  Memo3Read.Lines.Add(binstr(bufByteData[1],8));
end;

procedure TForm1.btn52WriteClick(Sender: TObject);
begin
  // Write Addres & Data to I2C
  bufByteWrite[1] := CtrlByteChip52Write;
  bufByteWrite[2] := AddressHiByte;
  bufByteWrite[3] := AddressLoByte;
  bufByteWrite[4] := DataByte;

  i2c.writeBuf(bufByteWrite);

  // Memo2Send.Lines.Add(binstr(bufByteWrite[1],8)+binstr(bufByteWrite[2],8)+bufByteWrite[3],8)+binstr(bufByteWrite[4],8));
  // such long did not work in TMemo
  // so it is split in two parts

  Memo2Send.Lines.Add(binstr(bufByteWrite[1],8)+binstr(bufByteWrite[2],8));
  Memo3Read.Lines.Add(binStr(bufByteWrite[3],8)+binstr(bufByteWrite[4],8));
end;

procedure TForm1.btn53ReadClick(Sender: TObject);
begin

end;

procedure TForm1.btn53WriteClick(Sender: TObject);
begin

end;

procedure TForm1.btnI2CDestroyClick(Sender: TObject);
begin
  i2c.Destroy;
end;

procedure TForm1.btnCreateDeviceClick(Sender: TObject);
begin
  i2c := TI2C.Create(sDevice);          // '/dev/i2c-1'

  btnSetDevice.Enabled     := true;
  btnCreateDevice.Enabled  := false;

  btnI2CDestroy.Enabled := true;
  btnI2CFree.Enabled    := true;
end;

procedure TForm1.btnScanBusClick(Sender: TObject);
var
   AStringList : TStringList;
   AProcess    : TProcess;        // needs process unit
begin

  AProcess    := TProcess.Create(nil);
  AStringlist := TStringList.create;

  AProcess.commandline := 'i2cdetect -y 1';    // this executes a raspberry command
  // you can try this in a shell as well
  AProcess.Options     := AProcess.Options + [poWaitOnExit, poUsePipes];

  Aprocess.Execute;
  Astringlist.LoadFromStream(AProcess.Output);

  Memo1.Clear;
  Memo1.Lines.Add('  Active I2C-adresses shown in Hex');
  Memo1.Lines.Add('');
  Memo1.Lines.Add(AStringlist.Text) ;

  AStringList.Free;
  AProcess.Free;

end;

procedure TForm1.btnSetDeviceClick(Sender: TObject);
begin
  i2c.SetDevice($53);        // number in hex
  // for now set to 53, just to see if it access the second chip, or the first at $52, as later defined in the GUI?

  btn52Read.Enabled  := True;
  btn52Write.Enabled := True;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin

  if rbI2C_1.Checked then sDevice := '/dev/i2c-1'
    else                  sDevice := '/dev/i2c-0';

  // strSlaveAddress   := 52;


  CtrlByteChip52Write := %10100100;
  CtrlByteChip52Read  := %10100101;

  // future second chip
  CtrlByteChip53Write := %10100110;
  CtrlByteChip53Read  := %10100111;

  edtCtrByteWr52.text := binstr(CtrlByteChip52Write,8);
  edtCtrByteRd52.text := binstr(CtrlByteChip52Read,8);
end;

procedure TForm1.Label1Click(Sender: TObject);
begin

end;

procedure TForm1.Memo1Change(Sender: TObject);
begin

end;

procedure TForm1.rbI2C_1Change(Sender: TObject);
begin

end;

procedure TForm1.spnedtAddressHiChange(Sender: TObject);
begin

  edtAddrHiBinWr.text := binstr(spnedtAddressHi.value,8);
  edtAddrHiBinRd.text := binstr(spnedtAddressHi.value,8);
  AddressHiByte := spnedtAddressHi.Value;
end;

procedure TForm1.spnedtAddressLoChange(Sender: TObject);
begin
  edtAddrLoBinWr.text := binstr(spnedtAddressLo.value,8);
  edtAddrLoBinRd.text := binstr(spnedtAddressLo.value,8);
  AddressLoByte := spnedtAddressLo.Value;
end;

procedure TForm1.spnedtDataChange(Sender: TObject);
begin
  edtDataBinWr.text := binstr(spnedtData.value,8);
  DataByte      := spnedtData.Value;
end;

end.

